// project created on 6/3/2004 at 8:34 PM
/*
 *   Copyright (c) 2004, Alexandros Frantzis (alf82 [at] freemail [dot] gr)
 *
 *   This file is part of Bless.
 *
 *   Bless is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Bless is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Bless; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Threading;
using System.IO;
using Gtk;
using Glade;
using Bless;
using Bless.Gui;
using Bless.Gui.Dialogs;

public class BlessMain
{
		[Glade.Widget] Gtk.HBox DataViewBox;
		[Glade.Widget] Gtk.Toolbar MainToolbar;
		[Glade.Widget] Gtk.Window MainWindow;
		[Glade.Widget] Gtk.Statusbar OffsetStatusBar;
		[Glade.Widget] Gtk.Statusbar SelectionStatusBar;
		[Glade.Widget] Gtk.Statusbar MessageStatusBar;
		[Glade.Widget] Gtk.Statusbar OverwriteStatusBar;
		[Glade.Widget] Gtk.MenuItem EditUndoMenuItem;
		[Glade.Widget] Gtk.MenuItem EditRedoMenuItem;
		
		Timer StatusMessageTimer;
		
		bool fileHasChanged;
		
        public static void Main (string[] args)
        {
                new BlessMain (args);
        }

        public BlessMain (string[] args) 
        {
                Application.Init();

                Glade.XML gxml = new Glade.XML (null, "bless.glade", "MainWindow", null);
                gxml.Autoconnect (this);
               	StatusMessageTimer= new Timer(new TimerCallback(ClearStatusMessage), null, System.Threading.Timeout.Infinite, 0);
               		
	            DataView dv=new DataView();
				dv.OffsetBar=OffsetStatusBar;
				dv.SelectionBar=SelectionStatusBar;
               	DataViewBox.PackStart(dv);
               	
               	
               	// setup dataview
                ByteBuffer bb=null;
                
                if (args.Length>0)
                	bb=OpenFile(args[0]);
	                
	            if (bb==null) {
	            	bb=new ByteBuffer();
	            	MainWindow.Title="(Untitled) - Bless";
	            }
	            
	            bb.Changed+=OnBufferChanged;
               	dv.Buffer=bb;
               	
               	dv.OverwriteBar=OverwriteStatusBar;
               	
               	OnBufferChanged();
               	
               	DataViewBox.ShowAll();
            	Application.Run();
        }
        
        void DisplayStatusMessage(string message)
        {
			MessageStatusBar.Pop(0);	
			MessageStatusBar.Push(0, message);
			StatusMessageTimer.Change(5000, 0);        	        
        }
        
		void ClearStatusMessage(object o)
        {
			MessageStatusBar.Pop(0);	
			//MessageStatusBar.Push(0, message);
			StatusMessageTimer.Change(System.Threading.Timeout.Infinite, System.Threading.Timeout.Infinite);        	        
        }
        
        ///<summary>Handle ByteBuffer changes</summary>
        void OnBufferChanged()
        {
        	ByteBuffer bb=(DataViewBox.Children[0] as DataView).Buffer;
			
			// if state has not changed don't alter title
        	if (bb.HasChanged!=fileHasChanged) {    	
    	    	fileHasChanged=bb.HasChanged;
        	
    	    	// Add an asterisk to the title if file has changed
    	    	string filename=bb.Filename;
        	
    	    	if (filename==null || filename.Length==0)
    	    		filename="(Untitled)";
        	
    	    	if (fileHasChanged)
	        		MainWindow.Title= filename + " * - Bless";
        		else
        			MainWindow.Title= filename + " - Bless";	
        	}
        	
        	// gray Undo if there is no undo action 
        	if (bb.CanUndo)
        		EditUndoMenuItem.Sensitive=true;
        	else 
        		EditUndoMenuItem.Sensitive=false;
        		
        	// gray Redo if there is no redo action 		
        	if (bb.CanRedo)
        		EditRedoMenuItem.Sensitive=true;
        	else
        		EditRedoMenuItem.Sensitive=false;
        			
        }
        
        ///<summary>If file has changed ask whether to save it.
        /// Returns true if file saved or the user doesn't want to save it.</summary>
        private bool AskForSaveIfFileChanged()
        {
        	ByteBuffer bb=(DataViewBox.Children[0] as DataView).Buffer;
        	if (bb == null)
        		return true;
        		
			if (bb.HasChanged) {
				SaveConfirmationAlert sca=new SaveConfirmationAlert(bb.Filename, MainWindow);
				ResponseType res=(ResponseType)sca.Run();
				sca.Destroy();
				
				if (res== ResponseType.Ok) {
					return SaveFile(null, false);
				}
				else if (res==ResponseType.No){
					return true;
				}
				else /*if (res==ResponseType.Cancel)*/
					return false;
				
			}
			
			return true;
				
        }
        
        private ByteBuffer OpenFile(string filename)
        {
        		string fullPath=null;
        		
        		// get the full path
        		try {
        			fullPath=Path.GetFullPath(filename);	
        		}
        		catch(Exception ex) {
        			ErrorAlert ea=new ErrorAlert("Error opening file "+filename, ex.Message, MainWindow);
					ea.Run();
					ea.Destroy();
        		}
        		
        		// if we can't get full path return
        		if (fullPath==null)
        			return null;
        		
        		try {	
					ByteBuffer bb=new ByteBuffer(fullPath);
					DisplayStatusMessage("Loaded file '"+ fullPath +"'");
					MainWindow.Title=fullPath + " - Bless";
					bb.Changed+=OnBufferChanged;
					fileHasChanged=false;
					return bb;
				}
				catch(UnauthorizedAccessException ex) {
					ErrorAlert ea=new ErrorAlert("Error opening file '"+ fullPath+"'", "You do not have read permissions for the file you requested.", MainWindow);
					ea.Run();
					ea.Destroy();
				}
				catch(System.IO.FileNotFoundException ex) {
					ErrorAlert ea=new ErrorAlert("Error opening file '"+ fullPath+"'", "The file you requested does not exist.", MainWindow);
					ea.Run();
					ea.Destroy();
				}
				catch(System.IO.IOException ex) {
					ErrorAlert ea=new ErrorAlert("Error opening file '"+ fullPath+"'", ex.Message, MainWindow);
					ea.Run();
					ea.Destroy();
				}
				catch(System.ArgumentException ex) {
					ErrorAlert ea=new ErrorAlert("Error opening file '"+ fullPath+"'", ex.Message, MainWindow);
					ea.Run();
					ea.Destroy();
				}
				return null;
        
        }
        
        private bool SaveFile(string filename, bool forceSaveAs)
        {
        	ByteBuffer bb=(DataViewBox.Children[0] as DataView).Buffer;
        	
        	// if a filename is given or if it is not given but buffer
        	// has a filename, just save it
        	if ( (filename!=null) || (!forceSaveAs && filename==null && bb.Filename!=null && bb.Filename.Length!=0))
        		return SaveFileInternal(filename);
        		  
        	// otherwise prompt user for a name	  
        	Gtk.FileSelection fs=new Gtk.FileSelection("Save File As");
			bool done=false;
			bool fileSaved=true;
			
			do {
				ResponseType response = (ResponseType)fs.Run();
				fs.Hide();
				if (response == ResponseType.Ok) {
					// check to see whether file exists and prompt user to confirm
					if (File.Exists(fs.Filename)) {
						FileOverwriteAlert ea=new FileOverwriteAlert(fs.Filename, MainWindow);
						ResponseType response1 = (ResponseType)ea.Run();
						ea.Destroy();
						if (response1 == ResponseType.Ok) {
							SaveFileInternal(fs.Filename);
							done=true;
						} else
							done=false;
					}
					else{
						SaveFileInternal(fs.Filename);
						done=true;
					}
				}
				else {
					done=true;
					fileSaved=false;
				}
					
			} while(!done);
				
			fs.Destroy();	
			
			return fileSaved; 
        }
        
        private bool SaveFileInternal(string filename)
        {
        	DataView dv=(DataViewBox.Children[0] as DataView);
        	ByteBuffer bb=dv.Buffer;
        	string fullPath=null;
        		
        	// don't allow dataview events while saving
        	dv.Sensitive=false;	
        		
        	// get the full path
        	try {
        		if (filename == null) 
        			filename = bb.Filename; 
        		fullPath=Path.GetFullPath(filename);	
        	}
        	catch(Exception ex) {
        		ErrorAlert ea=new ErrorAlert("Error saving file "+filename, ex.Message, MainWindow);
				ea.Run();
				ea.Destroy();
        	}
        		
        	// if we can't get full path, return
        	if (fullPath==null)
        		return false;
       
        	SaveProgressDialog spd=null;
        	try {
        		// decide whether to Save or Save As
        		if (fullPath!=bb.Filename) {
        			string msg="Saving file as '"+fullPath+"'";
        			DisplayStatusMessage(msg+"...");
        			
        			spd=new SaveProgressDialog(msg);
					bb.SaveAs(fullPath, true, new ByteBuffer.SaveProgressDelegate(spd.Update));
					spd.Destroy();
					
					DisplayStatusMessage("The file has been saved as '" + fullPath +"'");
					MainWindow.Title=fullPath + "- Bless";
				}
				else {
					string msg="Saving file '"+fullPath+"'";
        			DisplayStatusMessage(msg+"...");
        			
        			spd=new SaveProgressDialog(msg);
					bb.Save(new ByteBuffer.SaveProgressDelegate(spd.Update));
					spd.Destroy();
						
					DisplayStatusMessage("The file '" + fullPath + "' has been saved");
				}
				// re-allow DataView events 
				dv.Sensitive=true;
				
				return true;
			}
			catch (UnauthorizedAccessException ex) {
				ErrorAlert ea=new ErrorAlert("Error saving file '"+ fullPath+"'", "You don't have write permission for the specified location.", MainWindow);
				ea.Run();
				ea.Destroy();			
			}
			catch (System.ArgumentException ex) {
				ErrorAlert ea=new ErrorAlert("Error saving file '"+ fullPath+"'", ex.Message, MainWindow);
				ea.Run();
				ea.Destroy();
			}
			catch (System.IO.IOException ex) {
				ErrorAlert ea=new ErrorAlert("Error saving file '"+ fullPath+"'", ex.Message, MainWindow);
				ea.Run();
				ea.Destroy();
			}
			catch(ByteBuffer.SaveInterruptedException ex) {
					
			}
				
			spd.Destroy();
				
			DisplayStatusMessage("The file '" + fullPath + "' has NOT been saved");
			
			// re-allow DataView events
			dv.Sensitive=true;
			
			return false;
        }
        
        ///<summary>Handle file->new command from menu</summary>
        public void OnFileNew(object o, EventArgs args) 
		{
			if (AskForSaveIfFileChanged()) {
				ByteBuffer bb=new ByteBuffer();
				bb.Changed+=OnBufferChanged;
				fileHasChanged=false;
				(DataViewBox.Children[0] as DataView).Buffer=bb;
				OnBufferChanged();
				MainWindow.Title="(Untitled) - Bless";
			}
		}
		
		///<summary>Handle file->open command from menu</summary>
		public void OnFileOpen(object o, EventArgs args) 
		{
			// save the previous file if user wants to 
        	if (AskForSaveIfFileChanged()==false)
        			return;
        			
			Gtk.FileSelection fs=new Gtk.FileSelection("Open File");
			
			ResponseType result = (ResponseType)fs.Run();
			fs.Hide();
			
			if (result == ResponseType.Ok) {
				ByteBuffer bb=OpenFile(fs.Filename);
				if (bb!=null) {
					(DataViewBox.Children[0] as DataView).Buffer.CloseFile();
					(DataViewBox.Children[0] as DataView).Buffer=bb;
					OnBufferChanged();
				}	
			}
			
			fs.Destroy();
		}
		
		///<summary>Handle file->save command from menu</summary>
		public void OnFileSave(object o, EventArgs args)
		{
			SaveFile(null, false);			
		}
		
		///<summary>Handle file->save as command from menu</summary>
		public void OnFileSaveAs(object o, EventArgs args)
		{
			SaveFile(null, true);
		}
		
		///<summary>Handle file->quit command from menu</summary>
		public void OnFileQuit(object o, EventArgs args) 
		{
			if (AskForSaveIfFileChanged())
				Application.Quit ();
		}
		
		///<summary>Handle edit->undo command from menu</summary>
		public void OnEditUndo(object o, EventArgs args) 
		{
			    (DataViewBox.Children[0] as DataView).Undo();
			    (DataViewBox.Children[0] as DataView).Redraw();
		}
		
		///<summary>Handle edit->redo command from menu</summary>
		public void OnEditRedo(object o, EventArgs args) 
		{
			    (DataViewBox.Children[0] as DataView).Redo();
			    (DataViewBox.Children[0] as DataView).Redraw();
		}
		
		///<summary>Handle edit->cut command from menu</summary>
		public void OnEditCut(object o, EventArgs args) 
		{
		    (DataViewBox.Children[0] as DataView).Cut();
		}
		
		///<summary>Handle edit->copy command from menu</summary>
		public void OnEditCopy(object o, EventArgs args) 
		{
			(DataViewBox.Children[0] as DataView).Copy();
		}
		
		///<summary>Handle edit->paste command from menu</summary>
		public void OnEditPaste(object o, EventArgs args) 
		{
			(DataViewBox.Children[0] as DataView).Paste();
		}
		
		///<summary>Handle edit->delete command from menu</summary>
		public void OnEditDelete(object o, EventArgs args) 
		{
			(DataViewBox.Children[0] as DataView).Delete();
		}
		
		///<summary>Handle the View->Layouts command</summary>
		public void OnViewLayouts(object o, EventArgs args)
		{
			// show the selection dialog
			new LayoutSelectionDialog(DataViewBox.Children[0] as DataView);
		}
		
        ///<summary>Handle Main Window Delete Event</summary>
        public void OnMainWindowDeleteEvent (object o, DeleteEventArgs args) 
        {
        	if (AskForSaveIfFileChanged())
        		Application.Quit ();
           	args.RetVal = true;
        }
}

